function [nrm_new] = gennormextract(nrm, raw, parameters); 

% Deconvolution of Generalised SCIDAR covariance normalisation
% to extract the normalisation term required for the correct
% normalisation of the generalised SCIDAR secondary peaks.
%
% Inputs:  nrm        -   Overlapped covariance normalisation term
%          raw        -   Unnormalised scintillation covariance
%          parameters -   A vector containing values for 
%                         [zenith, binary_sep, m_diff, dr, scidar_defocus]
%                         where: zenith         - angle from the vertical (degrees)
%                                binary_sep     - separation of binary star
%                                                 components (arcsecs)
%                                m_diff         - magnitude difference of binary
%                                                 star components
%                                dr             - pupil plane sampling (m)
%                                scidar_defocus - amount of defocus beneath 
%                                                 telescope (km)
% Outputs: nrm_new   -    Extracted normalisation term
%
% Usage: [nrm_new] = gennormextract(nrm, raw, parameters);
%
% Rachel Johnston, 21/1/2002

% Preliminaries
zenith = parameters(1);
binary_sep = parameters(2);
m_diff = parameters(3);
dr = parameters(4);
scidar_defocus = parameters(5);

secz = 1/cos(zenith*pi/180); 		
thi = binary_sep/((3600*180)/pi);
alpha = 10^(m_diff/2.5);
asize=128;

%precomputed double stars rotation on ccd; (sgn(X),Y/|X|)
rdb='';
	if ((parameters(2)==5.7) & (parameters(3)==0)) %36OPH
   	rdb =[1,26/24];
   end
	if ((parameters(2)==8.2) & (parameters(3)==1.1)) % HIP13847
      rdb =[-1,59/9];
   end
	if ((parameters(2)==5.6) & (parameters(3)==0.5)) %ALFCRU
      rdb=[1,24/5];
   end

%Visualisation help: Create an unbiased mask for secondary peak detection
mask=raw(:,asize+1:asize*2);
r_mask = asize-1;
centralmed(1:r_mask)=median(raw(:,asize+1:r_mask+asize),1);
central=raw(:,asize+1:r_mask+asize);
for i=1:r_mask
   central(:,i)=centralmed(i);
end
mask(:,1:r_mask)=abs(raw(:,asize+1:r_mask+asize)-central(:,1:r_mask));


figure(1)
subplot (2,2,1)
imagesc(raw(:,129:256))
title ('Raw correlation');
subplot (2,2,2)
imagesc(abs(log10(1+mask)));grid
title ('Background removed');
xlabel('y coordinates')
ylabel('x coordinates')

%Now check direction of double star separation (automatic for known objects):
if (isempty (rdb)) 
	x = input('Input the x coordinate of the secondary peak closest to the central peak from the image in FIGURE 1  ');
	y = input('Input the y coordinate of the same peak');
else
   x=round(asize+1+asize/8*rdb(1));
   y=round(asize/8*rdb(2));
end

x_length = asize+1 - x;
y_length = y - 1;

% Check the values match the problem at hand.
offset = thi*secz*scidar_defocus*1000;
offset = round(offset/dr);
calc_offset = (sqrt(x_length^2 + y_length^2));
%if (abs(calc_offset - offset) > 5)
%    disp('Entered coordinates do not match calculated offset');
%    figure(1)
%    imagesc(raw(:,129:256))
%    xlabel('y coordinates')
%    ylabel('x coordinates')
%    x = input('Input the x coordinate of the secondary peak closest to the central peak from the image in FIGURE 1  ')
%    y = input('Input the y coordinate of the secondary peak closest to the central peak from the image in FIGURE 1  ')
%    x_length = 129 - x;
%    y_length = y - 1;    
%   calc_offset = (sqrt(x_length^2 + y_length^2));
%end 

fdim = max([x_length y_length])*2 + 4;
f = ((1+alpha^2)/alpha)*impulse(fdim/2 + 1, fdim/2 + 1, fdim, fdim) + impulse(fdim/2 + 1 - x_length, fdim/2 + 1 + y_length, fdim, fdim) + impulse(fdim/2 + 1 + x_length, fdim/2 + 1 - y_length, fdim, fdim); 


s=sprintf('Calculated offset %d pixels', offset);
disp (s);
s=sprintf('Offset from entered coordinates %d pixels', round(calc_offset));
disp (s);

figure(1)
subplot(2,2,3)
imagesc(raw(129-fdim/2:129+fdim/2-1,129:129+fdim/2-1))
subplot(2,2,4)
imagesc(f(:,fdim/2+1:fdim))



dim = 256;
hdim = dim - fdim;
h_pack = zeros(dim);
f_pack = zeros(dim);
f_pack(1:max(size(f)),1:max(size(f))) = f;
ft_f = fft2(f_pack);
g = nrm(1:dim-1,1:dim-1);

iterations = 100;
reg_value = 0;
cg_step = 1;
reg_matrix = reg_value*ones(dim);
error_matrix = ones(dim);

gs = zeros(dim);
gs(1:dim-1,1:dim-1)=g;
hs = zeros(dim);
hs(1:dim-max(size(f)),1:dim-max(size(f))) = g(129-hdim/2:129+hdim/2-1,129-hdim/2:129+hdim/2-1);
fs = zeros(dim);
fs(1:max(size(f)),1:max(size(f))) = f;

gm = zeros(dim);
hm = zeros(dim);
fm = zeros(dim);
gm(1:255,1:255) = ones(255);
hm(1:dim-max(size(f)),1:dim-max(size(f))) = ones(dim-max(size(f)));
fm(1:max(size(f)),1:max(size(f))) = ones(max(size(f)));	

hest4 = hs;
error_old4 = cgerror(hest4, fs, gs, gm, reg_matrix, error_matrix);
error_old4 = error_old4 + sum(sum(reg_matrix.*(fs.^2)));
clear fftcount;
fftcount = 0;
fftcount4 = 0;
errors4 = error_old4;

h = waitbar(0,'Please wait...');
for j = 1:iterations
    
    clear error_f error_h
	maskh = cgmask(gs, hest4, fs, gm, hm, fm, reg_matrix);
	[hnew4, errora] = cgrad(hest4, fs, gs, maskh, fm, gm, reg_matrix, error_matrix, cg_step);
	[hest4, lscount] = cgplsbls(hnew4, hest4, error_old4, fs, gs, gm, reg_matrix, reg_matrix, error_matrix);
	pos_errorh4 = cgerror(hest4, fs, gs, gm, reg_matrix, error_matrix);
	pos_errorh4 = pos_errorh4 + sum(sum(reg_matrix.*(fs.^2)));
	
	error_old4 = pos_errorh4;
	errors4 = [errors4 pos_errorh4];
	
	if j==1
		fftcount = fftcount + 3 + 2 + 2 + cg_step*2 + lscount*2;
	else
		fftcount = fftcount + 2 + cg_step*2 + lscount*2;
	end	
	fftcount4 = [fftcount4 fftcount];
    waitbar(j/iterations)
 end	
 close(h) 

nrm_new = zeros(256);
%nrm_new(129-hdim/2-x_length:129+hdim/2-1-x_length,129-hdim/2+y_length:129+hdim/2-1+y_length) = hest4(1:hdim,1:hdim); 
nrm_new = hest4(1:hdim, 1:hdim);
