function [Cn2, ro] = SCIDARProcessGen2ASCII(data, no_frames, filename, filename2, parameters, reg, flag);

% Performs generalised SCIDAR processing but does not account for
% dark count measurements.
%
% Inputs:   data       - Raw data filename, e.g. Run1_0.dat
%           no_frames  - Ensemble size, typically 2000
%           filename   - Name of file covariance results written to
%           filename2  - Name of file normalisation results written to
%           parameters - A vector containing values for 
%                        [zenith, binary_sep, m_diff, dr, scidar_defocus]
%                        where: zenith         - angle from the vertical (degrees)
%                               binary_sep     - separation of binary star
%                                                components (arcsecs)
%                               m_diff         - magnitude difference of binary
%                                                star components
%                               dr             - pupil plane sampling (m)
%                               scidar_defocus - amount of defocus beneath 
%                                                telescope (km)
%           reg        - regularisation parameter value (between 0 and 1)
%
% Outputs:  Cn2        - Estimated Cn2 profile 
%           ro         - Estimated ro value
%
% Usage: [Cn2, ro] = SCIDARProcessGen2('Run1_0.dat', 2000, 'Run1_0_auto', 'Run1_0_nrm', parameters, 0.01);
%
% Rachel Johnston, 23/1/2002
% Crude modification by Mo 12/07/02 to write Cn2 + height data to ASCII file for export.
% Fetch data/time of end of run
%
cd C:\SCIDAR\matlab\Temp;
filename1 = strcat(filename,'.mat');
s = which (filename1);
if (isempty(s))
d = which (data);
	if (isempty(d))
   error ('Requested Data File not existing in Scidar_data directory');   
   end
   
% Generating the autocovariance from the measured scintillation frames
	disp('Generating scintillation covariance');
	if flag=='fftw'
    	[raw, ac, av, nrm] = processdata2002(data, no_frames);
 	else
    	[raw, ac, av, nrm] = processdata2001Mo(data, no_frames);
	end
	save(filename, 'raw', 'ac', 'av', 'nrm');
	disp('Scintillation covariance calculated');
else
   load(filename, 'raw', 'ac', 'av', 'nrm');
   disp('Covariance retrieved from previous processing');
end 

figure (3)
subplot (2,2,1)
imagesc(abs(log10(1+raw)));grid
title ('Autocorrelation from 2000 images @375Hz');
labl=strcat('Zen,Sep,dMag=',num2str(parameters(1:3),'%5.1f'));
xlabel ([labl]);
labl=strcat('dR(cm)=',num2str(100*parameters(4),'%5.2f'),', dFoc(km)=',num2str(-parameters(5),'%5.2f'));
ylabel ([labl]);

% Extracting the appropriate normalisation term
filename1 = strcat(filename2,'.mat');
s = which (filename1);
if (isempty(s))
	disp('Extracting normalisation term');
	[nrm_new] = gennormextract(nrm, raw, parameters);
	save(filename2, 'nrm_new');
	disp('Normalisation term obtained');
else
	load(filename2, 'nrm_new');
   disp('Normalisation term retrieved from previous processing');
end

% Performing the pupil plane inversion
[Cn2, Cn2_unscaled, ro, h, Tmod, S] = SCIDARResultsnewgen(parameters, reg, filename, filename2);

% Referencing altitude to ground level
h=h-parameters(5)*10^(3);
ns_h = size(Cn2,1);
dh=h(2)-h(1);

% Displaying the results
figure (3)
subplot (2,2,2)
plot(h(1:size(Cn2,1))/1000,Tmod*Cn2_unscaled);
xlim ([-5,15])
hold on
plot(h(1:size(Cn2,1))/1000,S,'r')
xlabel('altitude abv ground (km)')
ylabel('covariance')
hold off
title('data(red), model(blue)');

%
figure (3)
subplot (2,2,3)
plot(h(1:size(Cn2,1))/1000,Cn2/dh);
xlim ([0,20])
xlabel('altitude abv ground (km)')
ylabel('Cn2 (m^{-2/3})')
titl=strcat('Profile',data(4:size(data,2)-4),': r0 (cm)=',num2str(ro*100,'%5.1f'));
title([titl]);
disp (titl);

% Computing integral Cn2.dh (m^1/3) for comparison with MASS
for i = 1:6
   cmass(i)=0.;
   hmass(i)=2^(i-2)*1000;
   hmin = hmass(i)/sqrt(2);
   hmax = 2*hmin;
   for j = 1:ns_h
      if ((h(j) >= hmin-dh/2) & (h(j) < hmin))
         cmass(i)=cmass(i)+ Cn2(j)*(h(j)+dh/2-hmin)/dh;
      end
      if ((h(j) >= hmin) & (h(j)<hmax))
         cmass(i)=cmass(i)+ Cn2(j);
      end
    	if ((h(j) >= hmax) & (h(j) < hmax+dh/2))
       cmass(i)=cmass(i)+ Cn2(j)*(hmax-h(j)+dh/2)/dh;
    	end
 	end
 end
 %triangular:
for i = 1:6
   tmass(i)=0.;
   hmass(i)=2^(i-2)*1000;
   hmin = hmass(i)/2;
   hmax = 4*hmin;
   for j = 1:ns_h
      if ((h(j) > hmin) & (h(j) <= hmass(i)))
         tmass(i)=tmass(i)+ Cn2(j)*(h(j)-hmin)/hmin;
      end
      if ((h(j) > hmass(i)) & (h(j)<=hmax))
         tmass(i)=tmass(i)+ Cn2(j)*(1-(h(j)-hmass(i))/hmass(i));
      end
    end
end
disp('Data for MASS comparison');
disp (cmass);
disp (tmass)

figure (3)
subplot (2,2,4)
bar(log10(hmass),cmass*10^13);
xlabel('log10 altitude (m) abv ground')
ylabel('Cn2dh (10^-13 m^{1/3})')
title ('MASS equivalent layers')
hold on
plot(log10(hmass),tmass*10^13,'+r');
hold off



% ASCII write extension Mo 12/07/02
% Export to txt file with name as run.txt
%Check stored as vertical vector (Matlab is a bit funny about this)

if size(Cn2,1) >1
    Cn2=transpose(Cn2);
end
if size(h,1) >1
    h=transpose(h);
end

   cd C:\SCIDAR\matlab\Results;
name_base=data(1:size(data,2)-4);
ext='.txt';
file=strcat(name_base,ext); 
out=[h;Cn2*10^13];
fid = fopen(file,'w');
fprintf(fid,'SCIDAR profile/dh (*10^-13 m^1/3): \n');
fprintf(fid,'%5.2f  %12.8f \n',out);
fprintf(fid,'r0= %5.2f m \n',ro);
out=[hmass;cmass*10^13;tmass*10^13];
fprintf(fid,'MASS triangular equivalent slabs -linear, triangular- (*10^-13 m^1/3): \n');
fprintf(fid,'%5.0f %12.8f %12.8f\n',out);
fprintf(fid,'Instrumental configuration (zen(deg),sep("),dM,dr(m),defoc(km)) \n');
fprintf(fid,'%5.2f %5.2f %5.2f %5.4f %5.2f\n',parameters);
fclose(fid);
disp('result file stored');
% NB These scripts have been lumped together for speed but
% there is some cleaning up to do due to sloppy implementation
