function [Cn2, ro] = SCIDARProcess2(data, no_frames, filename, parameters, reg, flag);

% Performs pupil plane SCIDAR processing but does not account for
% dark count measurements.
%
% Inputs:   data       - Raw data filename, e.g. Run1_0.dat
%           no_frames  - Ensemble size, typically 2000
%           filename   - Name of file covariance results written to
%           parameters - A vector containing values for 
%                        [zenith, binary_sep, m_diff, dr, scidar_defocus]
%                        where: zenith         - angle from the vertical (degrees)
%                               binary_sep     - separation of binary star
%                                                components (arcsecs)
%                               m_diff         - magnitude difference of binary
%                                                star components
%                               dr             - pupil plane sampling (m)
%                               scidar_defocus - amount of defocus beneath 
%                                                telescope (km)
%           reg        - regularisation parameter value (between 0 and 1)
%
% Outputs:  Cn2        - Estimated Cn2 profile 
%           ro         - Estimated ro value
%
% Usage: [Cn2, ro] = SCIDARProcess2('Run1_0.dat', 2000, 'Run1_0_auto', parameters, 0.01);
%
% Rachel Johnston, 23/1/2002
% Crude modification by Mo 12/07/02 to write Cn2 + height data to ASCII file for export.


% Generating the autocovariance from the measured scintillation frames
disp('Generating scintillation covariance');
if flag=='fftw'
    [raw, ac, av, nrm] = processdata2002(data, no_frames);
else
    [raw, ac, av, nrm] = processdata2001Mo(data, no_frames);
end
save(filename, 'raw', 'ac', 'av', 'nrm');
disp('Scintillation covariance calculated');


% Performing the pupil plane inversion
[Cn2, Cn2_unscaled, ro, h, Tmod, S] = SCIDARResultsnew(parameters, reg, filename);

% Displaying the results
h=h-parameters(5)*10^(3);
titl=strcat('r0=',num2str(ro,'%5.3f'));
figure
title(titl)
plot(h(1:size(Cn2,1)),Cn2)
xlabel('altitude (m)')
ylabel('Cn2.dh (m^{1/3})')
figure
plot(h(1:size(Cn2,1)),Tmod*Cn2_unscaled)
title('Fit to the data')
hold on
plot(h(1:size(Cn2,1)),S,'r')
xlabel('altitude (m)')
ylabel('covariance')
hold off

% ASCII write extension Mo 12/07/02
% Export to txt file with name as run.txt
CnScale=Cn2.*10^14; 
%Check stored as vertical vector (Matlab is a bit funny about this)

if size(CnScale,1) >1
    CnScale=transpose(CnScale);
end

if size(h,1) >1
    h=transpose(h);
end

cd C:\SCIDAR\matlab\Results;

name=data(1:size(data,2)-3);
ext='txt';
file=strcat(name,ext);

out=[h;CnScale];
fid = fopen(file,'w');
fprintf(fid,'%5.2f  %12.8f\n',out);
fclose(fid);




