function [profile, profile_unscaled, ro_est, h, Tmod, S] = scidarResultsnew(parameters, reg, filename);

% Modified Pupil Plane Processing.  Scaling is performed on the T 
% matrix and not on the calculated autocovariance, i.e. this function
% solves the following equation for Cn2
%           T' x Cn2 + n = S'
% where:    T'  - scaled T matrix
%           Cn2 - turbulence profile
%           n   - noise
%           S'  - unnormalised scintillation covariance
%
% Inputs:   parameters  -   A vector containing values for 
%                           [zenith, binary_sep, m_diff, dr, scidar_defocus]
%                           where: zenith         - angle from the vertical (degrees)
%                                  binary_sep     - separation of binary star
%                                                   components (arcsecs)
%                                  m_diff         - magnitude difference of binary
%                                                   star components
%                                  dr             - pupil plane sampling (m)
%                                  scidar_defocus - amount of defocus beneath 
%                                                   telescope (km)
%           reg         -   Regularisation parameter value (between 0 and 1)
%           filename    -   Name of MATLAB file containing covariance data
%
% Outputs:  profile     -   Estimated Cn2 profile (m^(-2/3))
%           ro          -   Estimated ro value (m)
%
% Usage:    [profile, ro] = scidarResultsnew([10, 15, 1, 0.01, 2.5], 0.1, 'run20_auto');
%
% Rachel Johnston, 21/1/2002

% Preliminaries
zenith = parameters(1);
binary_sep = parameters(2);
m_diff = parameters(3);
dr = parameters(4);
scidar_defocus = parameters(5);

secz = 1/cos(zenith*pi/180); 		
thi = binary_sep/((3600*180)/pi);
alpha = 10^(m_diff/2.5);
dh = 1*(dr/(thi*secz));
wavelength = 500e-9;		    
k = 2*pi/wavelength;

h1 = dh/2;                  % First altitude sample (m)
h_max = 25e3;               % Maximum range for calculations (m)
ns_h = ceil(h_max/dh);      % No. of altitude samples
if ns_h > 128
    ns_h  = 128;
    h_max = dh*ns_h;
end
sprintf(' Maximum height attainable is %5.2f meters',h_max)

h = h1:dh:ns_h*dh-h1;       % Altitude sample values (m)
r1=dr/2;
ns_r = ns_h ;               % No. of radius samples
r_max = thi*max(h)*secz ;   % Maximum radius corresponding to TRUE max height
r = r1:dr:ns_r*dr-r1;       % Radius sample values (m)

% Load autocorrelation data
load(filename)

% Extracting autocovariance slice
disp('Extracting covariance slice');
[S, binary_pos] = autoslice_2001(raw, ns_h, h, scidar_defocus, dr, zenith, binary_sep, alpha);

scale = max(max(nrm));
nrm = nrm/scale;
S = S/scale;
disp('Covariance slice extracted');

% Generating the kernel (Scaled T matrix in this case)
disp('Generating the inversion kernel');
[Tfull] = genTmat(zenith, binary_sep, dr, scidar_defocus, r, h);
weight_temp = autosliceweight_2001(nrm, ns_h, h, scidar_defocus, dr, zenith, binary_sep, binary_pos);
weight_temp(128)=0;
weight = zeros(size(Tfull));
[Trows, Tcols] = size(Tfull);
for i = 1:Tcols
   weight(:,i) = weight_temp(1:Trows)';
end  
Tmod = Tfull.*weight;
disp('Inversion kernel complete');

% The Inversion
disp('Performing the inverse problem');
[profile, error] = scicongradqp(Tmod, S', zeros(size(h')), reg, max(size(S)));
profile = profile.*(profile>0);
profile_unscaled = profile;
profile = profile*1e-14*dh*secz^(11/6);
Cn2_est = trapz(profile);
ro_est = (0.423*(k^2)*secz*Cn2_est)^(-3/5);
disp('Processing complete');