function[fest, errornew] = scicongradqp(H, g, finitial, lambda, iterations) 

% Inversion of 
%                   T' x Cn2 + n = S'
% for Cn2, using an accelerated quadratic programming technique which
% also enforces positivity (R. A. Johnston, T. J. Connolly and R. G. Lane,
% `An improved method for deconvolving a positive image', Opt. Commun.,
% Vol. 181, pp 267 - 278, 2000.)  
%
% Inputs:  T'         - SCIDAR kernel, a matrix of theoretical covariance curves
%          S'         - Column vector containing experimental covariance data 
%          Cn2_start  - Starting column vector estimate for Cn2
%		   lambda 	      - Regularisation parameter value
%          iterations - Number of conjugate gradient iterations to perform
% 
% Outputs: Cn2_est    - Column vector estimate of Cn2
%          error      - Goodness of fit = (T'xCn2-S')'x(T'xCn2-S')
%
% Usage:   [Cn2_est, error] = scicongradqp(Tmod, S', zeros(size(h')), 0.1, max(size(S)));
%
% Rachel Johnston, 21/1/2002

% Preliminaries
cgstep = 20;                    % 20 conjugate gradient steps per conjugate gradient iteration
Q = H'*H + lambda*eye(size(H'*H));
b = H'*g;


% The solution is only valid over pixels that are positive or may go positive.
% The following mask calculates these pixels.
fest = finitial;
fest = fest.*(fest>0);
grad = Q*fest-b;
d = -grad;

errorold = sum((H*fest - g).^2) + sum(lambda*fest.^2);

finished = 0;
finished2 = 0;
i = 1;
h = waitbar(0,'Inverting. Please wait...');
for i = 1:iterations
   
   
   	mask = ((fest>0)|(grad<0));
		fest = fest.*mask;
      grad = Q*fest-b;
      grad = grad.*mask;
		d = -grad;
      
      fold = fest;
      errorold = sum((H*fest - g).^2) + sum(lambda*fest.^2);
   	
      for j = 1:cgstep
         
         factor = d'*Q*d;
    		if (factor ~= 0)
         		grad = Q*fest-b;
         		alpha = -grad'*d/factor;

					fest = fest + alpha*d;
					fest = fest.*mask;
               grad = Q*fest-b;
               grad = grad.*mask;
               beta = grad'*Q*d/factor;
               d = -grad+beta*d;

          end
       end
       
      %Projected Linesearch to ensure E decreases and solution is positive
      errornew2 = 1e37;
      fnew = fest;
		direction = fnew-fold;
		alpha = 1;
		count = 0;
		while ((errornew2 > errorold)&(alpha > 1e-8))
			fnew = fold + alpha*(direction);
			fnew = fnew.*(fnew>0);
			errornew2 = sum((H*fnew - g).^2) + sum(lambda*fnew.^2);
			alpha = alpha/2;
			count = count + 1;
      end
      
      if alpha < 1e-8
         fest = fold;
      else
         fest = fnew;
      end 
      
      errornew(i) = sum((H*fest - g).^2) + sum(lambda*fest.^2);
     
 % computation here %
    waitbar(i/iterations)

     
      
   end

close(h) 